/**
 *
 * Copyright (c) 2010, Zed A. Shaw and Mongrel2 Project Contributors.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 * 
 *     * Neither the name of the Mongrel2 Project, Zed A. Shaw, nor the names
 *       of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _register_h
#define _register_h

#include <time.h>
#include <stdint.h>
#include <bstring.h>
#include <sys/types.h>
#include "task/task.h"

#define MAX_REGISTERED_FDS  64 * 1024
#define DEFAULT_MIN_PING 120
#define DEFAULT_MIN_READ_RATE 300
#define DEFAULT_MIN_WRITE_RATE 300
#define DEFAULT_KILL_LIMIT 2


extern uint32_t THE_CURRENT_TIME_IS;

typedef struct Registration {
    struct Connection *data;
    Task *task;
    uint16_t fd;
    uint32_t id;
    uint32_t last_ping;
    off_t last_read;
    off_t last_write;
    off_t bytes_read;
    off_t bytes_written;
} Registration;

int Register_connect(int fd, struct Connection *data);

int Register_disconnect(int fd);

int Register_ping(int fd);

int Register_read(int fd, off_t bytes);

int Register_write(int fd, off_t bytes);

void Register_init();

struct Connection *Register_fd_exists(int fd);

uint32_t Register_id_for_fd(int fd);

int Register_fd_for_id(uint32_t id);

int Register_cleanout();

void Register_destroy();

struct tns_value_t *Register_info();

#define Register_valid(R) ((R) != NULL && (R)->data != NULL)

#endif
